package org.acme;

import io.quarkus.test.junit.QuarkusTest;
import io.restassured.http.ContentType;
import jakarta.ws.rs.core.MediaType;
import org.junit.jupiter.api.Test;

import java.util.UUID;

import static io.restassured.RestAssured.given;
import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.is;

@QuarkusTest
class GreetingResourceTest {

    @Test
    void testHelloEndpoint() {

        MyEntity e = new MyEntity(UUID.randomUUID().toString(), "one", 0);

        e = given()
            .when()
            .contentType(MediaType.APPLICATION_JSON)
            .body(e)
            .log().all()
            .put("/entity")
            .then()
            .log().all()
            .statusCode(200)
            .body("field", equalTo("one"))
            .body("version", equalTo(0))
            .extract().response().jsonPath().getObject(".", MyEntity.class)
        ;

        e.setField("two");

        e = given()
            .when()
            .contentType(MediaType.APPLICATION_JSON)
            .body(e)
            .log().all()
            .post("/entity")
            .then()
            .log().all()
            .statusCode(200)
            .body("field", equalTo("two"))
            .body("version", equalTo(1))
            .extract().response().jsonPath().getObject(".", MyEntity.class)
        ;


        // NO CHANGE -> version should remain at 1, but becomes 2!

        given()
            .when()
            .contentType(MediaType.APPLICATION_JSON)
            .body(e) // <<< post the result of the last call
            .log().all()
            .post("/entity")

                /*
                Request method:	POST
                Request URI:	http://localhost:8081/entity
                Content-Type=application/json
                Body:
                {
                    "id": "5f8d86b2-2529-4d07-8497-bedab2df3799",
                        "field": "two",
                        "version": 1
                }

                select
                me1_0.ID,
                        me1_0.FIELD,
                        me1_0.VERSION
                from
                T_ENTITY me1_0
                where
                me1_0.ID=?

                2024-03-29 20:26:05,894 TRACE [org.hib.orm.jdb.bind] (executor-thread-1) binding parameter (1:VARCHAR) <- [5f8d86b2-2529-4d07-8497-bedab2df3799]

            This is generated but not necessary:

                update
                        T_ENTITY
                set
                    FIELD=?,
                    VERSION=?
                where
                    ID=?
                    and VERSION=?

                2024-03-29 20:26:05,895 TRACE [org.hib.orm.jdb.bind] (executor-thread-1) binding parameter (1:VARCHAR) <- [two]
                2024-03-29 20:26:05,895 TRACE [org.hib.orm.jdb.bind] (executor-thread-1) binding parameter (2:BIGINT) <- [2]
                2024-03-29 20:26:05,895 TRACE [org.hib.orm.jdb.bind] (executor-thread-1) binding parameter (3:VARCHAR) <- [5f8d86b2-2529-4d07-8497-bedab2df3799]
                2024-03-29 20:26:05,895 TRACE [org.hib.orm.jdb.bind] (executor-thread-1) binding parameter (4:BIGINT) <- [1]
                 */

            .then()
            .log().all()

                /*
                HTTP/1.1 200 OK
                content-length: 71
                Content-Type: application/json;charset=UTF-8

                {
                    "id": "5f8d86b2-2529-4d07-8497-bedab2df3799",
                        "field": "two",
                        "version": 2      <<<<<<<< UNNECESSARY!!!
                }
                 */

            .statusCode(200)
            .body("field", equalTo("two"))
            .body("version", equalTo(1)) // <<<< FAILS:
            //    java.lang.AssertionError: 1 expectation failed.
            //    JSON path version doesn't match.
            //    Expected: <1>
            //    Actual: <2>
        ;


    }

}